(*
Copyright 2009 Christian Sternagel

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
module F = Format;;
open Ceta;;

let string2cs s =
 let rec string2cs acc i =
  if i < 0 then acc else string2cs (String.get s i::acc) (i-1)
 in
 string2cs [] (String.length s - 1)
;;

let cs2string cs =
 let buf = Buffer.create (List.length cs) in
 List.iter (Buffer.add_char buf) cs;
 Buffer.contents buf
;;

let rec rev_append xs ys = match xs with
  | []    -> ys
  | x::xs -> rev_append xs (x::ys)
;;

let open_close f filename = try
  let chin = open_in filename in
  let result = f chin in
  close_in_noerr chin;
  result
with Sys_error msg -> (
  F.eprintf "%s\n%!" msg;
  exit(1);
)
;;

let contents = open_close (fun chin ->
  let len    = in_channel_length chin in
  let result = String.create len in
  let pos    = ref 0 in
  let loop   = ref true in
  while !loop do
    let n = input chin result !pos (min 1024 (len - !pos)) in
    if n = 0 then loop := false
             else pos := !pos + n
  done;
  string2cs(result)
);;


let start problem = match Ceta.certifyProof problem with
  | Ceta.Certified true  -> F.printf "CERTIFIED YES\n%!"
  | Ceta.Certified false -> F.printf "CERTIFIED NO\n%!"
  | Ceta.Error msg       -> (
    F.eprintf "%s\n%!" (cs2string msg);
    exit(1)
  )
;;

let args = Array.length(Sys.argv) in
if args = 2 then (
  let cs = contents(Sys.argv.(1)) in
  start cs
) else (
  F.eprintf "usage: %s problemFile.xml(in cpf-format)     (version 1.08)\n%!" Sys.argv.(0);
  exit(1)
)
